# -*- coding: utf-8 -*-
try:
    import re2 as re
except ImportError:
    import re


PLACEHOLDER = '<@!hmin_placeholder_%s_!@>'
RE_REPLACED_TAG = re.compile(
    ur'(<(script|textarea|style|pre).*?>.*?</(script|textarea|style|pre)>)', re.S
)
RE_COMMENTS = re.compile(ur'<!--(?!\[if.*?\]).*?-->', re.S | re.I)


def minify(content, remove_comments=True):
    # helpers
    def tag_replace(m):
        key = PLACEHOLDER % m.start()
        safe_storage[key] = m.group(1)
        return key

    # decode unicode
    try:
        content = content.decode('utf-8')
    except (UnicodeDecodeError, UnicodeEncodeError):
        pass

    # replace dangerous tags with placeholders
    safe_storage = dict()
    content = RE_REPLACED_TAG.sub(tag_replace, content)

    # fast version of "\s{2,} to \s"
    content = ' '.join(content.split())
    # "strip_space_between_tags"
    content = content.replace('> <', '><')
    if remove_comments:
        content = RE_COMMENTS.sub('', content)

    # and return dangerous tags back
    for key, value in safe_storage.items():
        content = content.replace(key, value)

    return content



# -*- coding: utf-8 -*-
from django.conf import settings

from .base import minify


""" Minification html middleware
"""
ENABLED = getattr(settings, 'HTML_MINIFY', not settings.DEBUG)
REMOVE_COMMENTS = getattr(settings, 'HMIN_REMOVE_COMMENTS', True)
EXCLUDE = []


if hasattr(settings, 'HMIN_EXCLUDE'):
    for url_pattern in settings.HMIN_EXCLUDE:
        regex = re.compile(url_pattern)
        EXCLUDE.append(regex)


class MarkMiddleware(object):
    def process_request(self, request):
        request.need_to_minify = True


class MinMiddleware(object):
    def process_response(self, request, response):
        # prevent from minifying cached pages
        if not hasattr(request, 'need_to_minify') or\
           not request.need_to_minify:
            return response

        # prevent from minifying excluded pages
        if EXCLUDE:
            path = request.path.lstrip('/')
            for regex in EXCLUDE:
                if regex.match(path):
                    return response

        if 'Content-Type' in response and\
           'text/html' in response['Content-Type'] and ENABLED:
            response.content = minify(response.content, REMOVE_COMMENTS)
        return response
